/*
 * File: Crossword.js
 * ------------------
 * This program displays a numbered crossword grid in the graphics window.
 */

"use strict";

/* Constants */

const GWINDOW_WIDTH = 640;
const GWINDOW_HEIGHT = 640;
const SQUARE_SIZE = 40;
const LABEL_FONT = "12px Helvetica";
const LABEL_DX = 3;
const LABEL_DY = 14;

const SAMPLE_GRID = [
   "      #    #   ",
   "      #    #   ",
   "           #   ",
   "       #       ",
   "   #   #       ",
   "       #     ##",
   "       #       ",
   "   #########   ",
   "       #       ",
   "##     #       ",
   "       #   #   ",
   "       #       ",
   "   #           ",
   "   #    #      ",
   "   #    #      "
];
function Crossword() {
   let gw = GWindow(GWINDOW_WIDTH, GWINDOW_HEIGHT);
   drawNumberedCrosswordGrid(gw, createCrosswordGrid(SAMPLE_GRID));
}

/*
 * Draws a numbered crossword grid on the graphics window.  The grid
 * parameter is a two-dimensional grid of booleans indicating the
 * black squares.
 */
function drawNumberedCrosswordGrid(gw, grid) {
   let height = grid.length;
   let width = grid[0].length;
   let x0 = (gw.getWidth() - width * SQUARE_SIZE) / 2;
   let y0 = (gw.getHeight() - height * SQUARE_SIZE) / 2;
   let number = 1;
   for (let i = 0; i < height; i++) {
      let y = y0 + i * SQUARE_SIZE;
      for (let j = 0; j < width; j++) {
         let x = x0 + j * SQUARE_SIZE;
         let sq = GRect(x, y, SQUARE_SIZE, SQUARE_SIZE);
         sq.setFilled(grid[i][j]);
         gw.add(sq);
         if (isSquareNumbered(grid, i, j)) {
            let label = GLabel("" + number);
            label.setFont(LABEL_FONT);
            gw.add(label, x + LABEL_DX, y + LABEL_DY);
            number++;
         }
      }
   }
}

/*
 * Returns true if the grid square at [i][j] should be numbered.
 * A crossword square is numbered if it is at the beginning of
 * a word, either across or down.  In the horizontal dimension,
 * a square is the beginning of a word if:
 *
 * 1. It is a white square.
 * 2. The square to its left is a black square or off the grid.
 * 3. The square to its right is a white square.
 *
 * The vertical dimension is symmetrical.
 */
function isSquareNumbered(grid, i, j) {
   return !isBlackSquare(grid, i, j) &&
          ((isBlackSquare(grid, i - 1, j) && !isBlackSquare(grid, i + 1, j)) || 
           (isBlackSquare(grid, i, j - 1) && !isBlackSquare(grid, i, j + 1))); 
}

/*
 * Returns true if the grid square at [i][j] is either a black square or
 * off the grid.
 */
function isBlackSquare(grid, i, j) {
   if (i < 0 || i >= grid.length) return true;
   if (j < 0 || j >= grid[0].length) return true;
   return grid[i][j];
}

/*
 * Creates a boolean grid from an array of strings.  White squares
 * appear at any position in the grid containing a space; any other
 * character indicates a black square.
 */
function createCrosswordGrid(lines) {
   let height = lines.length;
   let width = lines[0].length;
   let grid = [ ];
   for (let i = 0; i < height; i++) {
      let row = [ ];
      for (let j = 0; j < width; j++) {
         row.push(lines[i].charAt(j) !== ' ');
      }
      grid.push(row);
   }
   return grid;
}
